#!/usr/bin/env python3
import logging
import sys
from functools import partial

import requests
from pydm import Display
from pydm.widgets.channel import PyDMChannel
from qtpy import QtCore, QtWidgets
from qtpy.QtGui import QPixmap

from siriushlacon.mbtemp.consts import (
    BO_PIC1,
    BO_PIC2,
    BO_PIC3,
    BO_PIC4,
    BOEXTRACTION_PIC,
    BOINJ_PIC,
    CNPEM_LOGO,
    LNLS_LOGO,
    MBTEMP_MAIN_UI,
    OVERVIEW_MAIN,
    PIC_LA,
    PIC_P7RF,
    PIC_PA,
    SR_PICS,
    SRINJ_PIC,
)

logger = logging.getLogger()

TAB = {0: "RF", 1: "TB", 2: "TS", 3: "BO", 4: "SI", 5: "LA", 6: "PA"}

DEVICES_IP = {  # {Area IPs:[ip, devices (To specific devices only)]}
    "RF": [["10.128.102.119", [None, None]], ["10.128.101.118", [None, None]]],
    "TB": [["10.128.119.106", [5, 6]]],
    "TS": [["10.128.120.106", [0, 4]], ["10.128.101.117", [3, 4]]],
    "LA": [["10.128.122.102", [None, None]]],
    "PA": [["10.128.123.131", [None, None]], ["10.128.123.132", [None, None]]],
    "BO": [["10.128.1{:0>2d}.106", [None, None]]],
    "SI": [
        ["10.128.1{:0>2d}.117", [None, None]],
        ["10.128.1{:0>2d}.118", [None, None]],
    ],
}


class MBTempMonitoring(Display):
    def __init__(self, parent=None, macros=None, args=None):
        super().__init__(
            parent=parent, args=args, macros=macros, ui_filename=MBTEMP_MAIN_UI
        )
        self.OverviewButton.filenames = [OVERVIEW_MAIN]
        self.OverviewButton.openInNewWindow = True

        self.tab = ""
        self.addr = []
        self.url = "http://10.0.38.42:26001/devices"

        self.P7RF.setPixmap(QPixmap(PIC_P7RF))
        self.BOInj.setPixmap(QPixmap(BOINJ_PIC))
        self.SRInj.setPixmap(QPixmap(SRINJ_PIC))
        self.LinacArea.setPixmap(QPixmap(PIC_LA))
        self.PowerArea.setPixmap(QPixmap(PIC_PA))
        self.LogoLNLS.setPixmap(QPixmap(LNLS_LOGO))
        self.LogoCNPEM.setPixmap(QPixmap(CNPEM_LOGO))
        self.BOExtraction.setPixmap(QPixmap(BOEXTRACTION_PIC))

        for SRimg in range(1, 8):
            getattr(self, "SR_Pic{}".format(SRimg)).setPixmap(QPixmap(SR_PICS[SRimg]))

        self.tabWidget.currentChanged.connect(self.sector_change_connect)
        self.sector.valueChanged.connect(self.sector_change_connect)

        self.sector_change_connect()

    def sector_change_connect(self):
        self.tab = TAB[self.tabWidget.currentIndex()]
        self.sector_change_disconnect()

        channels = []
        self.board_sensors = {}

        logger.info("Area {}; Sector {}".format(self.tab, self.sector.value()))

        try:
            info_request = requests.get(
                self.url, verify=False, params={"type": "mbtemp"}, timeout=5
            )
        except Exception:
            QtWidgets.QMessageBox.warning(
                self, "Warning", "Impossible connect to {}".format(self.url)
            )
            logger.warning("Impossible connect to {}".format(self.url))
            sys.exit()
        dev = info_request.json()

        if self.tab in ["RF", "TB", "TS", "LA", "PA"]:
            self.sector.setEnabled(False)

        elif self.tab == "BO":
            self.sector.setSingleStep(2)
            self.sector.setEnabled(True)
            self.sector.setMaximum(19)

            if self.sector.value() % 2 == 0:
                self.sector.setValue(1)

            sectorFrom = 2 + (self.sector.value() // 2) * 5
            sectorTo = 7 + (self.sector.value() // 2) * 5

            for x, y in enumerate(range(sectorFrom, sectorTo)):
                if y != 51:
                    getattr(self, "BO_Sec_{}".format(x + 1)).setText(
                        "Booster Sector: {}".format(y)
                    )
                else:
                    self.BO_Sec_5.setText("Booster Sector: 1")
            self.setBOImage(sectorFrom)
        else:
            self.sector.setSingleStep(1)
            self.sector.setEnabled(True)
            self.sector.setMaximum(20)

        for ip in DEVICES_IP[self.tab]:  # dict -> {MBTemp:[CH1,CH2...]}
            for board in dev[ip[0].format(self.sector.value())][ip[1][0] : ip[1][1]]:
                for enabled in range(1, 9):
                    channels.append(board["channels"]["CH{}".format(enabled)]["prefix"])
                self.board_sensors[board["prefix"]] = channels
                channels = []

        for mbtemp in self.board_sensors:
            for coef in ["Alpha", "LinearCoef-Mon", "AngularCoef-Mon"]:
                slot = partial(
                    self.update_mbtemp,
                    pvname=mbtemp,
                    coef=coef,
                    sector=self.sector.value(),
                )
                mb = PyDMChannel(
                    address="ca://{}:{}".format(mbtemp, coef),
                    value_slot=slot,
                    connection_slot=slot,
                )
                self.addr.append(mb)
                mb.connect()

            for number, pv in enumerate(self.board_sensors[mbtemp]):
                slot = partial(
                    self.update_channel,
                    name_pv=pv,
                    mbtemp_name=mbtemp,
                    mbtemp_ch=number + 1,
                )
                temp = PyDMChannel(
                    address="ca://" + pv, value_slot=slot, connection_slot=slot
                )
                self.addr.append(temp)
                temp.connect()

    def update_channel(self, value, name_pv, mbtemp_name, mbtemp_ch):

        chosenArea = self.tab
        _id = mbtemp_name[-2:]

        try:
            if chosenArea == "TS":
                if _id == "14":
                    chosenArea = "SI"
                aux = _id

            elif chosenArea == "BO" and mbtemp_ch < 4 and mbtemp_name != "TB-MBTemp":
                location = int(_id) % 5
                aux = location if location != 0 else 5

            elif chosenArea in ["SI", "PA", "LA"]:
                aux = _id
                if (mbtemp_ch == 3) and (
                    mbtemp_name in ["SI-01-MBTemp-13", "SI-20-MBTemp-23"]
                ):
                    aux += "_2"

            elif chosenArea == "RF":
                aux = _id
                if _id == "23":
                    aux += "_2"
                    chosenArea = "SI"

            elif chosenArea == "TB":
                if mbtemp_name[:-3] == "BO-MBTemp":
                    return ()
                aux = ""

            else:
                return ()
            channel = getattr(self, "{}{}_Ch{}".format(chosenArea, aux, mbtemp_ch))

        except AttributeError or ValueError:
            return ()

        # ----------- Update color and toolTip ----------
        if value is False or value < 1:
            channel.brush = QtCore.Qt.red
            channel.setToolTip(
                ("PV: {}\n" + "Channel: {}\n" + "Process Variable Disconnected").format(
                    name_pv, mbtemp_ch
                )
            )
            return ()
        elif value is True:
            channel.brush = QtCore.Qt.green
            return ()
        elif value > self.tempMax.value():
            if (value > 410) and (value < 425):
                channel.brush = QtCore.Qt.cyan
                channel.setToolTip(
                    ("PV: {}\n" + "The Channel {} of MBTemp {} is open!").format(
                        name_pv, mbtemp_ch, mbtemp_name
                    )
                )
                return ()
            else:
                channel.brush = QtCore.Qt.yellow
        else:
            channel.brush = QtCore.Qt.green

        channel.setToolTip(
            (
                "Temperature: {} °C\n" + "MBTemp: {}\n" + "Channel: {}\n" + "PV: {}"
            ).format(value, mbtemp_name, mbtemp_ch, name_pv)
        )

    def update_mbtemp(self, value, pvname, coef, sector):
        last_Value = []
        new_val = {"Alpha": 0, "Linear Coefficient": "", "Angular Coefficient": ""}
        coefficients = {
            "Alpha": "Alpha",
            "LinearCoef-Mon": "Linear Coefficient",
            "AngularCoef-Mon": "Angular Coefficient",
        }

        try:
            id_addr = int(pvname[-2:])
        except Exception:
            if self.tab != "TB":
                return ()
            else:
                mb = 1

        if self.tab == "BO":
            location = id_addr % 5
            mb = location if location != 0 else 5

        elif self.tab in ["TS", "SI", "PA", "LA", "RF"]:
            mb = id_addr
        try:
            mbtemp = getattr(self, "{}_MBTemp{:0>2d}".format(self.tab, mb))
        except AttributeError:
            return ()

        if value is False:
            mbtemp.brush = QtCore.Qt.red
            mbtemp.setToolTip(("{} - MBTemp Board is disconnected").format(pvname))
            return ()
        elif value is True:
            mbtemp.brush = QtCore.Qt.blue
            return ()
        mbtemp.brush = QtCore.Qt.blue

        last_Value = mbtemp.toolTip().split("\n")

        for val in last_Value[1:]:
            new_val[val.split(": ")[0]] = val.split(": ")[1]

        new_val[coefficients[coef]] = value

        mbtemp.setToolTip(
            (
                "MBTemp: {}\n"
                + "Alpha: {}\n"
                + "Linear Coefficient: {}\n"
                + "Angular Coefficient: {}"
            ).format(
                pvname,
                new_val["Alpha"],
                new_val["Linear Coefficient"],
                new_val["Angular Coefficient"],
            )
        )

    def sector_change_disconnect(self):
        for channel in self.addr:
            channel.disconnect()
        if self.tab != "SI":
            return ()

        for disc in [
            "11_Ch4",
            "11_Ch5",
            "11_Ch6",
            "13_Ch8",
            "22_Ch6",
            "13_Ch3",
            "10_Ch1",
            "10_Ch2",
            "23_Ch8",
            "11_Ch2",
            "21_Ch1",
            "22_Ch4",
            "23_Ch3",
            "23_2_Ch3",
            "23_Ch4",
            "13_2_Ch3",
            "_MBTemp10",
        ]:
            getattr(self, "SI{}".format(disc)).brush = QtCore.Qt.gray
            getattr(self, "SI{}".format(disc)).setToolTip("")

    def setBOImage(self, secFrom):
        if secFrom % 2 == 0:
            self.BO_Img1.setPixmap(QPixmap(BO_PIC3))
            self.BO_Img2.setPixmap(QPixmap(BO_PIC2))
            self.BO_Img3.setPixmap(QPixmap(BO_PIC3))
            self.BO_Img4.setPixmap(QPixmap(BO_PIC1))
            self.BO_Img5.setPixmap(QPixmap(BO_PIC3))
        else:
            self.BO_Img1.setPixmap(QPixmap(BO_PIC1))
            self.BO_Img2.setPixmap(QPixmap(BO_PIC4))
            self.BO_Img3.setPixmap(QPixmap(BO_PIC1))
            self.BO_Img4.setPixmap(QPixmap(BO_PIC3))
            self.BO_Img5.setPixmap(QPixmap(BO_PIC1))


if __name__ == "__main__":
    app = QtWidgets.QApplication(sys.argv)
    window = MBTempMonitoring()
    window.show()
    sys.exit(app.exec_())



import datetime
import logging
import typing

import pytz
import requests

from siriushlacon.epics import Alarm, Severity
from siriushlacon.utils.consts import ARCHIVER_URL, SP_TZ, TIME_ZERO

logger = logging.getLogger()


def get_data_from_archiver(
    pv: str,
    from_: datetime.datetime,
    to: typing.Optional[datetime.datetime] = None,
    fetch_latest_metadata: bool = True,
):
    """
    :param fetch_latest_metadata: if true, an extra call is made to the engine as part of the retrieval to get the
        latest values of the various fields (DESC, HIHI etc).
    :param pv: PV name
    :param from_: initial time UTC
    :param to: final time, defaults to the current time UTC
    :return: request HTTP response
    """
    if not from_:
        raise ValueError('cannot complete request, "_from" is not defined')

    from_str = ""
    if type(from_) == datetime.datetime:
        logger.debug('converting "from_" datetime.time to str')
        from_str = get_time_str_from_utc(from_)

    to_str = ""
    if to and type(to) == datetime.datetime:
        logger.debug('converting "to" datetime.time to str')
        to_str = get_time_str_from_utc(to)
    if not to:
        to_str = datetime.datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%S.%f")[:-3] + "Z"

    params = {
        "pv": pv,
        "to": to_str,
        "from": from_str,
        "fetchLatestMetadata": fetch_latest_metadata,
    }

    req = requests.Request("GET", url=ARCHIVER_URL, params=params)
    prepared = req.prepare()
    logger.info("prepared request {} {}".format(prepared.method, prepared.url))
    return requests.Session().send(prepared)


def get_time_str_from_utc(utc_time: datetime.datetime):
    """
    :param utc_time: datetime object
    :return: string formatted timestamp
    """
    if not utc_time.tzinfo:
        logger.warning(
            'parameter "utc_time" does not have a timezone defined so UTC time will be assumed'
        )
    else:
        if utc_time.utcoffset() != TIME_ZERO:
            logger.debug('parameter "utc_time" being converted to UTC time')
            utc_time = utc_time.astimezone(pytz.utc)
    return utc_time.strftime("%Y-%m-%dT%H:%M:%S.%f")[:-3] + "Z"


def print_entry(reading):
    # Timestamp
    print(datetime.datetime.fromtimestamp(reading["secs"]).astimezone(SP_TZ))
    # Severity
    print("Severity " + Severity.nameOf(reading["severity"]))
    # Alarm Status
    print("Alarm " + Alarm.nameOf(reading["status"]))

    # Handle Value
    print(reading["val"])
    value = reading["val"]

    # Considering a meaning per bit ...
    msgs = []
    map_ = {0: "Zero", 1: "One", 2: "Two"}
    for k, v in map_.items():
        if value & 1 << k:
            msgs.append(v)
    print("{}".format(msgs))

    print("\n")


if __name__ == "__main__":

    initial_time = datetime.datetime(2019, 1, 1, 0, 0, 0).astimezone(SP_TZ)
    response = get_data_from_archiver(
        pv="BO-01U:VA-SIP20-BG:SetErrorCode-Mon", from_=initial_time
    )
    if response.status_code == 200:
        for data in response.json()[0]["data"]:
            print_entry(data)
    else:
        print("invalid status code")



#!/usr/bin/env python3
import json

from pydm import Display
from pydm.widgets.byte import PyDMByteIndicator
from pydm.widgets.related_display_button import PyDMRelatedDisplayButton
from qtpy.QtGui import QColor
from qtpy.QtWidgets import QLabel

from siriushlacon import __date__, __version__
from siriushlacon.vbc.consts import MAIN_WINDOW_UI, SYSTEM_WINDOW_PY
from siriushlacon.widgets.images import CNPEM_PIXMAP, LNLS_PIXMAP


class VBCMainWindow(Display):
    def __init__(self, parent=None, args=None, macros=None):
        super(VBCMainWindow, self).__init__(
            parent=parent, args=args, macros=macros, ui_filename=MAIN_WINDOW_UI
        )

        self.lnlsLabel: QLabel
        self.labelVersion: QLabel
        self.cnpemLabel: QLabel

        self.VBC_car_1: PyDMRelatedDisplayButton
        self.VBC_car_2: PyDMRelatedDisplayButton
        self.VBC_car_3: PyDMRelatedDisplayButton
        self.VBC_car_4: PyDMRelatedDisplayButton
        self.VBC_car_5: PyDMRelatedDisplayButton
        self.VBC_car_6: PyDMRelatedDisplayButton
        self.VBC_car_7: PyDMRelatedDisplayButton
        self.VBC_car_8: PyDMRelatedDisplayButton
        self.VBC_car_9: PyDMRelatedDisplayButton
        self.VBC_car_10: PyDMRelatedDisplayButton

        self.LED_1: PyDMByteIndicator
        self.LED_2: PyDMByteIndicator
        self.LED_3: PyDMByteIndicator
        self.LED_4: PyDMByteIndicator
        self.LED_5: PyDMByteIndicator
        self.LED_6: PyDMByteIndicator
        self.LED_7: PyDMByteIndicator
        self.LED_8: PyDMByteIndicator
        self.LED_9: PyDMByteIndicator
        self.LED_10: PyDMByteIndicator

        self.lnlsLabel.setPixmap(LNLS_PIXMAP)
        self.lnlsLabel.setFixedSize(LNLS_PIXMAP.size())

        self.cnpemLabel.setPixmap(CNPEM_PIXMAP)
        self.cnpemLabel.setFixedSize(CNPEM_PIXMAP.size())

        self.labelVersion.setText(f"{__version__} {__date__}")

        self.VBC_car_1.filenames = [SYSTEM_WINDOW_PY]
        self.VBC_car_2.filenames = [SYSTEM_WINDOW_PY]
        self.VBC_car_3.filenames = [SYSTEM_WINDOW_PY]
        self.VBC_car_4.filenames = [SYSTEM_WINDOW_PY]
        self.VBC_car_5.filenames = [SYSTEM_WINDOW_PY]
        self.VBC_car_6.filenames = [SYSTEM_WINDOW_PY]
        self.VBC_car_7.filenames = [SYSTEM_WINDOW_PY]
        self.VBC_car_8.filenames = [SYSTEM_WINDOW_PY]
        self.VBC_car_9.filenames = [SYSTEM_WINDOW_PY]
        self.VBC_car_10.filenames = [SYSTEM_WINDOW_PY]

        self.VBC_car_1.macros = json.dumps({"IOC": "VBC1", "CAR": "1"})
        self.VBC_car_2.macros = json.dumps({"IOC": "VBC2", "CAR": "2"})
        self.VBC_car_3.macros = json.dumps({"IOC": "VBC3", "CAR": "3"})
        self.VBC_car_4.macros = json.dumps({"IOC": "VBC4", "CAR": "4"})
        self.VBC_car_5.macros = json.dumps({"IOC": "VBC5", "CAR": "5"})
        self.VBC_car_6.macros = json.dumps({"IOC": "VBC6", "CAR": "6"})
        self.VBC_car_7.macros = json.dumps({"IOC": "VBC7", "CAR": "7"})
        self.VBC_car_8.macros = json.dumps({"IOC": "VBC8", "CAR": "8"})
        self.VBC_car_9.macros = json.dumps({"IOC": "VBC9", "CAR": "9"})
        self.VBC_car_10.macros = json.dumps({"IOC": "VBC10", "CAR": "10"})

        COLOR_RED = QColor(255, 0, 0)
        self.LED_1._disconnected_color = COLOR_RED
        self.LED_2._disconnected_color = COLOR_RED
        self.LED_3._disconnected_color = COLOR_RED
        self.LED_4._disconnected_color = COLOR_RED
        self.LED_5._disconnected_color = COLOR_RED
        self.LED_6._disconnected_color = COLOR_RED
        self.LED_7._disconnected_color = COLOR_RED
        self.LED_8._disconnected_color = COLOR_RED
        self.LED_9._disconnected_color = COLOR_RED
        self.LED_10._disconnected_color = COLOR_RED



import logging
import time

from siriushlacon.vbc.epics import ACP, BBB, ProcessOff, ProcessOn, System, Turbovac

_logger = logging.getLogger(__name__)


class ProcessOffAction:
    def __init__(self, prefix: str):
        if not prefix:
            raise ValueError(f"parameter prefix cannot be empty {prefix}")
        self.prefix = prefix
        self._tick = 0.5

        self.process_on = ProcessOn(prefix=prefix)
        self.process_off = ProcessOff(prefix=prefix)
        self.bbb = BBB(prefix=prefix)
        self.turbovac = Turbovac(prefix=prefix)
        self.acp = ACP(prefix=prefix)
        self.system = System(prefix=prefix)

    def run(self):
        self._clear_pvs()
        self._stage_1()
        self._stage_2()
        self._stage_3()
        self._stage_4()
        self._stage_5()
        self._stage_6()

    def _clear_pvs(self):
        """clear all status PVs"""
        _logger.info("clear_pvs")
        self.process_off.clear_all_fv_status()
        self.process_on.clear_all_status()

    def _stage_1(self):
        _logger.info("stage1")
        # close pre-vacuum valve (and keeps gate valve open)
        self.turbovac.pre_vacuum_valve_sw_pv.value = 0

        # wait until valve receives command to open
        while self.turbovac.pre_vacuum_valve_sw_pv.value:
            time.sleep(self._tick)

        self.process_off.off_fv_status1_pv.value = 1

    def _stage_2(self):
        _logger.info("stage2")
        # change venting valve to manual control
        self.turbovac.ak_sp_pv.value = 0
        self.turbovac.pnu_sp_pv.value = 134
        self.turbovac.ind_sp_pv.value = 2
        self.turbovac.pwe_sp_pv.value = 18
        self.turbovac.ak_sp_pv.value = 7

        # turn TURBOVAC and ACP15 pumps OFF
        self.turbovac.pzd1_sp_zrvl_pv.value = 0
        self.acp.on_off_pv.value = 0

        # wait until pump receives command to turn off
        while self.acp.on_off_pv.value:
            time.sleep(self._tick)

        self.process_off.off_fv_status2_pv.value = 1

    def _stage_3(self):
        """wait until TURBOVAC frequency decrease to 600 Hz"""
        _logger.info("stage3")
        while self.turbovac.pzd2_rb_pv.value > self.system.off_frequency_pv.value:
            time.sleep(self._tick)

        self.process_off.off_fv_status3_pv.value = 1

    def _stage_4(self):
        _logger.info("stage4")
        # open X203 valve (TURBOVAC venting valve)
        self.turbovac.venting_valve_sw_pv.value = 1

        # update UI checkbox status
        self.turbovac.venting_valve_ui_pv_rval.value = 1

        # wait until venting valve receives command to close
        while self.turbovac.venting_valve_sw_pv.value == 0:
            time.sleep(self._tick)

        self.process_off.off_fv_status4_pv.value = 1

    def _check_bbb_less_than_off_pressure(self) -> bool:
        bbb_off_pressure = (
            self.system.off_pressure_base_pv.value
            * 10 ** self.system.off_pressure_exp_pv.value
        )
        return self.bbb.pressure_pv.value < bbb_off_pressure

    def _stage_5(self):
        _logger.info("stage5")
        while self._check_bbb_less_than_off_pressure():
            time.sleep(self._tick)

        self.process_off.off_fv_status5_pv.value = 1

    def _stage_6(self):
        """Stage 6:"""
        _logger.info("stage6")

        # close all the valves (gate valve is already closed)
        self.bbb.gate_valve_sw_pv.value = 0
        self.turbovac.venting_valve_sw_pv.value = 0  # close X203
        self.bbb.gate_valve_ui_pv.value = 0
        self.turbovac.venting_valve_ui_pv_rval.value = 0  # close X203

        # wait until venting valve receives command to close
        while self.bbb.gate_valve_sw_pv.value:
            time.sleep(self._tick)

        self.process_off.off_fv_status6_pv.value = 1

        # complement value of PV to launch "Process Finished" window
        self.process_off.toggle()


def process_off(prefix: str):
    """
    this script do all the procedures turn off the system with full ventilation
    it is divided in 6 stages, described as follow:
        -stage 1: close pre-vacuum valve (keep gate valve open)
        -stage 2: turn ACP15 and TURBOVAC pumps off
        -stage 3: wait TURBOVAC slowdown to 600 Hz
        -stage 4: open X203 (TURBOVAC venting valve)
        -stage 5: wait pressure decrease to 760 Torr
        -stage 6: close X203 and gate valves
    """
    action = ProcessOffAction(prefix=prefix)
    action.run()



import logging
import re

from qtpy.QtCore import QRegExp
from qtpy.QtGui import QIntValidator, QRegExpValidator
from qtpy.QtWidgets import QFrame, QGridLayout, QLabel, QLineEdit, QPushButton

logger = logging.getLogger()

MIN_SEC_PATTERN = r"^([0-5]?[0-9]):([0-5]?[0-9])$"
MIN_SEC_REG = re.compile(MIN_SEC_PATTERN)


class ParametersFrame(QFrame):
    def __init__(self, *args, **kwargs):
        super(ParametersFrame, self).__init__(*args, **kwargs)
        self.setFrameStyle(QFrame.Panel | QFrame.Raised)
        self.contentLayout = QGridLayout()

        self.setpointVoltageLabel = QLabel("Fixed voltage [3000 to 7000] V")
        self.setpointVoltageSettingLabel = QLabel()
        self.setpointVoltageInp = QLineEdit()
        self.setpointVoltageInp.setMaximumWidth(100)
        self.setpointVoltageInp.setValidator(QIntValidator(3000, 7000))
        self.setpointVoltageInp.setToolTip("Final voltage setpoint.")
        self.setpointVoltageInp.setText("3000")
        self.contentLayout.addWidget(self.setpointVoltageLabel, 0, 0, 1, 1)
        self.contentLayout.addWidget(self.setpointVoltageInp, 0, 1, 1, 1)
        self.contentLayout.addWidget(self.setpointVoltageSettingLabel, 0, 2, 1, 1)

        self.stepToFixDelayLabel = QLabel("Step to Fixed delay (mm:ss)")
        self.stepToFixDelaySettingLabel = QLabel()
        self.stepToFixDelayInp = QLineEdit()
        self.stepToFixDelayInp.setMaximumWidth(100)
        self.stepToFixDelayInp.setValidator(QRegExpValidator(QRegExp(MIN_SEC_PATTERN)))
        self.stepToFixDelayInp.setToolTip("Delay between toStep and toFixed calls.")
        self.stepToFixDelayInp.setText("05:00")

        self.contentLayout.addWidget(self.stepToFixDelayLabel, 1, 0, 1, 1)
        self.contentLayout.addWidget(self.stepToFixDelayInp, 1, 1, 1, 1)
        self.contentLayout.addWidget(self.stepToFixDelaySettingLabel, 1, 2, 1, 1)

        self.setButton = QPushButton("Confirm")
        self.setButton.clicked.connect(self.confirm)
        self.setButton.setToolTip("Apply settings")
        self.contentLayout.addWidget(self.setButton, 3, 1, 1, 1)

        self.setLayout(self.contentLayout)
        self.confirm()

    def confirm(self):
        try:
            voltageString = self.setpointVoltageInp.text()
            tmp = int(voltageString)
            self.voltage = 3000 if tmp < 3000 else (7000 if tmp > 7000 else tmp)
            self.setpointVoltageSettingLabel.setText("{} V".format(self.voltage))
        except ValueError:
            pass

        try:
            delayString = self.stepToFixDelayInp.text()
            if MIN_SEC_REG.match(delayString):
                res = MIN_SEC_REG.search(delayString)
                self.delay = 60 * float(res.group(1)) + float(res.group(2))
                self.stepToFixDelaySettingLabel.setText(
                    "{} mm:ss ({}s)".format(delayString, self.delay)
                )
        except ValueError:
            pass



#!/usr/bin/env python3
import platform
import sys

import pkg_resources
from setuptools import find_packages, setup

from src.siriushlacon import (
    GENERIC_LAUNCHER_FILE_NAME,
    MAIN_LAUNCHER_FILE_NAME,
    VBC_LAUNCHER_FILE_NAME,
    __author__,
    __version__,
)

assert sys.version_info >= (3, 6, 2), "siriushlacon requires Python 3.6.2+"


def get_abs_path(relative) -> str:
    return pkg_resources.resource_filename(__name__, relative)


def get_long_description() -> str:
    desc = ""
    with open(get_abs_path("README.md"), "r") as _f:
        desc += _f.read().strip()

    desc += "\n\n"

    with open(get_abs_path("CHANGES.md"), "r") as _f:
        desc += _f.read().strip()

    return desc


long_description = get_long_description()

with open(get_abs_path("requirements.txt"), "r") as _f:
    requirements = _f.read().strip().split("\n")

entry_style = "console_scripts" if platform.system() == "Windows" else "gui_scripts"
setup(
    name="siriushlacon",
    author=__author__,
    entry_points={
        entry_style: [
            f"{GENERIC_LAUNCHER_FILE_NAME}=siriushlacon_launcher:launch_generic",
            f"{MAIN_LAUNCHER_FILE_NAME}=siriushlacon_launcher:launch_main_window",
            f"{VBC_LAUNCHER_FILE_NAME}=siriushlacon_launcher:launch_vbc",
        ]
    },
    classifiers=[
        "Intended Audience :: Science/Research",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python",
        "Topic :: Scientific/Engineering",
    ],
    description="Client Applications for Sirius",
    download_url="https://github.com/lnls-sirius/pydm-opi",
    license="GNU GPLv3",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/lnls-sirius/pydm-opi/",
    project_urls={
        "Changelog": "https://github.com/lnls-sirius/pydm-opi/",
    },
    version=__version__,
    install_requires=requirements,
    include_package_data=True,
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    python_requires=">=3.6",
    scripts=[
        "scripts/sirius-hla-as-ap-bbb-monitor.py",
        "scripts/sirius-hla-as-ap-conlauncher.py",
        "scripts/sirius-hla-as-ap-countingpru.py",
        "scripts/sirius-hla-as-ap-mbtemp.py",
        "scripts/sirius-hla-as-ap-pctrl.py",
        "scripts/sirius-hla-as-ps-regatron-individual.py",
        "scripts/sirius-hla-as-ps-regatron.py",
        "scripts/sirius-hla-as-pu-spixconv.py",
        "scripts/sirius-hla-as-va-agilent4uhv-device.py",
        "scripts/sirius-hla-as-va-agilent4uhv.py",
        "scripts/sirius-hla-as-va-mks937b.py",
        "scripts/sirius-hla-as-va-vbc.py",
        "scripts/sirius-hla-bo-va-agilent4uhv-overview.py",
        "scripts/sirius-hla-bo-va-mks937b-overview.py",
        "scripts/sirius-hla-si-va-agilent4uhv-overview.py",
        "scripts/sirius-hla-si-va-mks937b-overview.py",
        "scripts/sirius-hla-tb-va-agilent4uhv-overview.py",
        "scripts/sirius-hla-tb-va-mks937b-overview.py",
        "scripts/sirius-hla-ts-va-agilent4uhv-overview.py",
        "scripts/sirius-hla-ts-va-mks937b-overview.py",
    ],
    zip_safe=False,
)



import logging
import sys

from aux_dialogs import AddPV, AddTeam, AddUser, EditItems, Login
from models import TableModel
from pydm import Display
from pymongo import MongoClient, errors
from PyQt5.QtGui import QKeySequence, QRegExpValidator
from qtpy.QtCore import QRegExp, Qt, QThread, Signal
from qtpy.QtWidgets import (
    QApplication,
    QComboBox,
    QItemDelegate,
    QMainWindow,
    QMessageBox,
    QShortcut,
)

from siriushlacon.epicstel.consts import EPICSTEL_HOST, EPICSTEL_MAIN_UI, EPICSTEL_TABS

logger = logging.getLogger()


class UpdateTableThread(QThread):
    finished = Signal(dict)

    def __init__(self, db):
        QThread.__init__(self)
        self.db = db

    def __del__(self):
        self.wait()

    def run(self):
        tables = {}
        for table in EPICSTEL_TABS[:3]:
            items = list(self.db[table].find())
            headers = list(items[0].keys())
            data = []

            for doc in items:
                row = []
                for header in headers:
                    item = doc[header]

                    if type(item) == list:
                        if len(item) and type(item[0]) == dict:
                            row.append(
                                ",".join(
                                    [
                                        "{} ({})".format(
                                            (list(i.values())[:2])[0],
                                            (list(i.values())[:2])[1],
                                        )
                                        for i in item
                                    ]
                                )
                            )
                        else:
                            row.append(",".join(item))
                    else:
                        row.append(item)
                data.append(row)

            tables[table] = (headers, data)

        self.finished.emit(tables)


class Delegate(QItemDelegate):
    def __init__(self, owner, choices):
        super().__init__(owner)
        self.items = choices

    def createEditor(self, parent, option, index):
        self.editor = QComboBox(parent)
        self.editor.addItems(self.items)
        return self.editor

    def setEditorData(self, editor, index):
        value = index.data(Qt.DisplayRole)
        num = self.items.index(value)
        editor.setCurrentIndex(num)

    def setModelData(self, editor, model, index):
        value = editor.currentText()
        model.setData(index, value, Qt.EditRole)

    def updateEditorGeometry(self, editor, option, index):
        editor.setGeometry(option.rect)


class Window(Display, QMainWindow):
    def open(self, credentials):
        client = MongoClient(EPICSTEL_HOST)

        try:
            client.admin.authenticate(credentials[0], credentials[1])
            client.server_info()
            self.db = client.epicstel

            self.update_thread = UpdateTableThread(self.db)
            self.update_thread.finished.connect(self.display_table)

            self.refresh_btn.clicked.connect(self.update_table)
            self.first = True
            self.update_table()
        except errors.OperationFailure:
            QMessageBox.critical(
                self,
                "Could not authenticate",
                "Invalid credentials",
                QMessageBox.Ok,
            )
            self.login_remote()

    def cell_clicked(self, args, tab):
        col = args.column()
        if tab == "pvs" and col == 2:
            self.pvs_table.setItemDelegateForColumn(
                2, Delegate(self.pvs_table.model(), list(self.db.pvs.distinct("group")))
            )
            self.pvs_table.openPersistentEditor(args)
        elif (tab == "users" and col not in [1, 3]) or (tab == "teams" and col != 1):
            model = getattr(self, tab + "_table").model()
            previous_data = list(filter(lambda x: x, model.data(args).split(",")))
            mongo_id = model.data(model.index(args.row(), 0))

            if (tab == "users" and col == 5) or (tab == "teams" and col == 2):
                pvs = [
                    "{} ({})".format(p.get("name"), p.get("group"))
                    for p in self.db.pvs.find()
                ]
                dialog = EditItems(mongo_id, previous_data, pvs, tab, "pvs")
            elif tab == "users" and col == 6:
                dialog = EditItems(
                    mongo_id,
                    previous_data,
                    self.db.teams.distinct("team"),
                    tab,
                    "teams",
                )
            elif tab == "users" and col == 2:
                dialog = EditItems(
                    mongo_id,
                    previous_data,
                    self.db.teams.distinct("team"),
                    tab,
                    "adminof",
                )
            else:
                dialog = EditItems(
                    mongo_id,
                    previous_data,
                    self.db.pvs.distinct("group"),
                    tab,
                    "groups",
                )

            dialog.edit_items.connect(self.edit_items)
            dialog.show()

    def edit_items(self, id, values, tab, field):
        if field == "pvs":
            pvs = {"pvs": []}
            for pv in values:
                splitted = pv.split(" ")
                name, group = splitted[0], splitted[1][1:-1]
                pv_id = self.db.pvs.find_one({"name": name, "group": group}).get("_id")
                pvs["pvs"].append({"name": name, "group": group, "ext_id": pv_id})

            self.db.users.update_one({"_id": id}, {"$set": pvs})
        else:
            self.db[tab].update_one({"_id": id}, {"$set": {field: values}})
        self.update_table()

    def del_ca_addr(self):
        for index in self.ca_addr_list.selectedIndexes():
            self.ca_addr_list.takeItem(index.row())

        self.update_ca_addr()

    def del_team(self, args):
        indexes = self.teams_table.selectedIndexes()
        model = self.teams_table.model()
        team_i = model.getHeaders().index("team")
        teams = list(set([model.getRow(index)[team_i] for index in indexes]))

        self.db.teams.delete_many({"team": {"$in": teams}})
        self.db.users.update_many(
            {"teams": {"$in": teams}}, {"$pullAll": {"teams": teams, "adminof": teams}}
        )

        self.update_table()

    def del_pv(self, args):
        indexes = self.pvs_table.selectedIndexes()
        model = self.pvs_table.model()
        group_i, name_i = model.getHeaders().index("group"), model.getHeaders().index(
            "name"
        )

        groups, pvs = [], []

        for index in indexes:
            row = model.getRow(index)
            groups.append(row[group_i])
            pvs.append(row[name_i])

        groups, pvs = list(set(groups)), list(set(pvs))

        self.db.pvs.delete_many({"group": {"$in": groups}, "name": {"$in": pvs}})
        self.db.users.update_many(
            {}, {"$pull": {"pvs": {"group": groups, "name": pvs}}}
        )
        self.db.teams.update_many(
            {}, {"$pull": {"pvs": {"group": groups, "name": pvs}}}
        )

        self.update_table()

    def del_user(self, args):
        indexes = self.users_table.selectedIndexes()
        model = self.users_table.model()
        ids = list(set([model.getRow(index)[0] for index in indexes]))

        self.db.users.delete_many({"_id": {"$in": ids}})
        self.update_table()

    def show_add_dialog(self, args):
        if args == "user":
            dialog = AddUser([t.get("team") for t in self.db.teams.find()])
        elif args == "pv":
            dialog = AddPV(self.db.pvs.distinct("group"))
        elif args == "team":
            dialog = AddTeam(
                [
                    "{} ({})".format(u.get("fullname"), u.get("chat_id"))
                    for u in self.db.users.find()
                ]
            )

        add_type = "add_" + args
        getattr(dialog, add_type).connect(lambda c: getattr(self, add_type)(c))
        dialog.show()

    def update_ca_addr(self):
        ips = [
            self.ca_addr_list.item(i).text() for i in range(self.ca_addr_list.count())
        ]
        self.db.configs.update_one(
            {"config": "EPICS_CA_ADDR_LIST"}, {"$set": {"ips": ips}}
        )
        self.update_table()

    def add_ca_addr(self, args):
        self.ca_addr_list.addItem(self.ca_addr_input.text())
        self.update_ca_addr()

    def add_user(self, args):
        if self.db.users.find_one({"chat_id": args[1]}):
            QMessageBox.critical(
                self,
                "Invalid Chat ID",
                "{} already exists in the user database".format(args[1]),
                QMessageBox.Ok,
            )
            self.show_add_dialog("user")
            return

        self.db.users.insert_one(
            {
                "chat_id": args[1],
                "pvs": [],
                "groups": [],
                "adminof": [],
                "teams": [args[2]],
                "fullname": args[0],
                "ignore": [],
            }
        )
        self.update_table()

    def add_pv(self, args):
        if self.db.pvs.find_one({"name": args[0], "group": args[4]}):
            QMessageBox.critical(
                self,
                "Invalid Name/Group",
                "{} already exists under the group {} in the PV database".format(
                    args[0], args[4]
                ),
                QMessageBox.Ok,
            )
            self.show_add_dialog("pv")
            return

        if args[2] < args[1] or args[3] <= 0:
            QMessageBox.critical(
                self,
                "Invalid Min/Max",
                "The maximum limit should be greater than the minimum limit",
                QMessageBox.Ok,
            )
            self.show_add_dialog("pv")
            return

        self.db.pvs.insert_one(
            {
                "name": args[0],
                "group": args[4],
                "max": args[2],
                "min": args[1],
                "timeout": args[3],
                "value": 0,
                "last_alert": 0,
                "d_time": 0,
                "d_count": 0,
            }
        )
        self.update_table()

    def add_team(self, args):
        if self.db.teams.find_one({"team": args[0]}):
            QMessageBox.critical(
                self,
                "Invalid Team Name",
                "{} team already exists".format(args[0]),
                QMessageBox.Ok,
            )
            self.show_add_dialog("team")
            return

        adm = args[1].split("(")
        adm[1] = int(adm[1][:-1])

        users = [[u.split("(")[0], int(u.split("(")[1][:-1])] for u in args[2]]
        users.append(adm)

        for user in users:
            self.db.users.update_one(
                {"chat_id": user[1]},
                {
                    "$setOnInsert": {
                        "pvs": [],
                        "groups": [],
                        "adminof": [],
                        "chat_id": user[1],
                        "fullname": user[0],
                    },
                    "$addToSet": {"teams": args[0]},
                },
                upsert=True,
            )

        self.db.teams.insert_one({"team": args[0], "pvs": [], "groups": []})
        self.db.users.update_one(
            {"chat_id": adm[1]}, {"$addToSet": {"adminof": args[0]}}
        )

        self.update_table()

    def login_remote(self):
        logger.info("Logging in...")
        login = Login()
        login.logged_in.connect(lambda c: self.open(c))

        login.show()

    def update_data(self, value, prop, id):
        col = EPICSTEL_TABS[self.tabWidget.currentIndex()]
        if not (
            (col == "users" and prop not in ["chat_id", "fullname"])
            or (col == "teams" and prop != "team")
        ):
            self.db[col].update_one({"_id": id}, {"$set": {prop: value}})
            self.update_table()

    def display_table(self, tables):
        for tab, table in tables.items():
            view = getattr(self, "{}_table".format(tab))
            model = TableModel(table[1], table[0])
            model.data_changed.connect(self.update_data)

            view.setModel(model)
            view.setColumnHidden(0, True)

        self.ca_addr_list.clear()
        self.ca_addr_list.addItems(
            self.db.configs.find_one({"config": "EPICS_CA_ADDR_LIST"}).get("ips")
        )

    def update_table(self):
        editing = (
            self.pvs_table.state() == 3
            or self.users_table.state() == 3
            or self.teams_table.state() == 3
        )
        if not self.update_thread.isRunning() and not editing:
            self.update_thread.start()

    def __init__(self, parent=None, macros=None, **kwargs):
        super().__init__(parent=parent, macros=macros, ui_filename=EPICSTEL_MAIN_UI)
        self.login_remote()

        self.delete_pv_btn.clicked.connect(self.del_pv)
        self.delete_user_btn.clicked.connect(self.del_user)
        self.delete_team_btn.clicked.connect(self.del_team)

        self.add_user_btn.clicked.connect(lambda: self.show_add_dialog("user"))
        self.add_pv_btn.clicked.connect(lambda: self.show_add_dialog("pv"))
        self.add_team_btn.clicked.connect(lambda: self.show_add_dialog("team"))

        self.pvs_table.clicked.connect(lambda c: self.cell_clicked(c, "pvs"))
        self.teams_table.clicked.connect(lambda c: self.cell_clicked(c, "teams"))
        self.users_table.clicked.connect(lambda c: self.cell_clicked(c, "users"))

        ip_regex = "^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]).){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$"

        self.ca_addr_input.setValidator(QRegExpValidator(QRegExp(ip_regex)))
        self.del_sc = QShortcut(QKeySequence("Del"), self)
        self.del_sc.activated.connect(self.del_ca_addr)
        self.add_ca_addr_btn.clicked.connect(self.add_ca_addr)


if __name__ == "__main__":
    app = QApplication(sys.argv)
    win = Window()
    win.show()
    sys.exit(app.exec_())



import pkg_resources


def get_abs_path(relative):
    return pkg_resources.resource_filename(__name__, relative)


TABLE_ALARMS_QSS: str = ""
with open(get_abs_path("resources/css/table-alarm.qss")) as f:
    TABLE_ALARMS_QSS = "".join(f.readlines())



import asyncio
import datetime
import logging

import requests
from pydm import Display
from qtpy.QtCore import QDateTime
from qtpy.QtWidgets import QTreeWidgetItem

from siriushlacon.epics import Alarm, Severity
from siriushlacon.regatron.consts import ALARM_UI, EXT_READINGS, STD_READINGS
from siriushlacon.utils.archiver import get_data_from_archiver
from siriushlacon.utils.consts import SP_TZ

logger = logging.getLogger()


class AlarmDisplay(Display):
    """
    Query alarms from the archiver appliance and display in a human readable format
    """

    @staticmethod
    def reading_tree_item(reading):
        """
        Get data from archiver and transform into a node with branches according to the bit value defined by the mapping.

        :param reading: A data item from the archiver request
        representing the bit position starting from zero and the key is a string with it's meaning
        :return: A QTreeWidgetItem
        """
        # Timestamp
        timestamp = str(
            datetime.datetime.fromtimestamp(reading["secs"]).astimezone(SP_TZ)
        )
        severity = Severity.nameOf(reading["severity"])
        alarm_status = Alarm.nameOf(reading["status"])
        value = reading["val"]

        node = QTreeWidgetItem(
            ["{} {} {} {}".format(timestamp, value, severity, alarm_status)]
        )

        for i in range(32):
            if value & 1 << i:
                node_child = QTreeWidgetItem(["{:X}".format(i)])
                node.addChild(node_child)

        return node

    def __init__(self, parent=None, macros=None, **kwargs):
        super().__init__(parent=parent, macros=macros, ui_filename=ALARM_UI)
        self.btnNow.clicked.connect(self.set_time_now)
        self.btnSearch.clicked.connect(self.search_alarms)

        # Tree Widget
        self.treeStdWarn.setColumnCount(1)
        self.treeStdWarn.setHeaderLabels(["Standard Warning"])

        self.treeStdErr.setColumnCount(1)
        self.treeStdErr.setHeaderLabels(["Standard Error"])

        self.treeExtWarn.setColumnCount(1)
        self.treeExtWarn.setHeaderLabels(["Extended Warning"])

        self.treeExtErr.setColumnCount(1)
        self.treeExtErr.setHeaderLabels(["Extended Error"])

        self.set_time_now()
        self.search_alarms()

    def get_PV(self, signal: str, std: bool = False, error: bool = False):
        """
        Build a PV name

        :param signal: Signal name, the last component of the PV
        :param std: True for 'Std' else 'Ext'
        :param error: True for 'Err' else 'Warn'
        :return: the PV name
        """
        return "{}:{}{}{}".format(
            self.macros()["P"],
            self.macros()["T"],
            "Err" if error else "Warn",
            signal,
        )

    def set_time_now(self):
        """Set current date and time to the "dfTo" widget"""
        self.dtTo.setDateTime(QDateTime.currentDateTime())

    def do_search(self, time_to, time_from, std, error):
        """
        Query and build the alarm tree

        :param time_to: final time
        :param time_from: initial time
        :param std: True for 'Std' else 'Ext', defines if it's a standard or extended alarm
        :param error: True for 'Err' else 'Warn', defines if it's an error or a warning
        """
        loop = asyncio.get_event_loop()
        futures = []
        # For each PV
        for signal in [*(STD_READINGS if std else EXT_READINGS)]:
            pv = self.get_PV(signal, std=std, error=error)
            futures.append(
                loop.run_in_executor(
                    None, get_data_from_archiver, pv, time_from, time_to, False
                )
            )
        return futures

    def update_alarm_tree(self, alarm_tree, response: requests.Response):
        """
        Build the alarm tree
        :param alarm_tree: QTreeWidget to append the alarms
        :param responses: typing.List[requests.Response]
        """

        # For each PV
        # for response in responses:
        if response.status_code != 200:
            logger.warning(
                "Invalid status code for request. Response {}".format(response)
            )
            logger.debug(response.text)
            return
        try:
            if len(response.json()) < 0:
                logger.info("empty response")
                return

            json_data = response.json()[0]

            pv_node = QTreeWidgetItem(["{}".format(json_data["meta"]["name"])])
            alarm_tree.addTopLevelItem(pv_node)
            for data in response.json()[0]["data"]:
                if data["val"] == 0:
                    # Will not display alarms, only actual value changes
                    continue

                pv_node_child = self.reading_tree_item(data)
                pv_node.addChild(pv_node_child)
        except Exception:
            logger.exception("Failed to add node")

    async def search(self):
        time_to = self.dtTo.dateTime().toPyDateTime().astimezone(SP_TZ)
        time_from = self.dtFrom.dateTime().toPyDateTime().astimezone(SP_TZ)
        async_responses = []

        async_responses.append(
            (
                self.treeStdWarn,
                self.do_search(
                    time_to=time_to,
                    time_from=time_from,
                    std=True,
                    error=False,
                ),
            )
        )
        async_responses.append(
            (
                self.treeStdErr,
                self.do_search(
                    time_to=time_to,
                    time_from=time_from,
                    std=True,
                    error=True,
                ),
            )
        )
        async_responses.append(
            (
                self.treeExtWarn,
                self.do_search(
                    time_to=time_to,
                    time_from=time_from,
                    std=False,
                    error=False,
                ),
            )
        )
        async_responses.append(
            (
                self.treeExtErr,
                self.do_search(
                    time_to=time_to,
                    time_from=time_from,
                    std=False,
                    error=True,
                ),
            )
        )
        for widget, futures in async_responses:
            widget.clear()
            for response in await asyncio.gather(*futures):
                self.update_alarm_tree(widget, response)

    def search_alarms(self):
        """Update all tree widgets"""
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self.search())
